@@ -549,7 +549,16 @@ class GSSAPIProxy(ContextProxy):
             log.debug("GSSAPI step input: %s", base64.b64encode(in_token or b"").decode())
 
         out_token = self._context.step(in_token)
-        self._context_attr = int(self._context.actual_flags)
+
+        try:
+            self._context_attr = int(self._context.actual_flags)
+        except gss_errors.MissingContextError:  # pragma: no cover
+            # MIT krb5 before 1.14.x will raise this error if the context isn't
+            # complete. We should only treat it as an error if it happens when
+            # the context is complete (last step).
+            # https://github.com/jborean93/pyspnego/issues/55
+            if self._context.complete:
+                raise
 
         if not self._is_wrapped:
             # When using NTLM through GSSAPI without it being wrapped by SPNEGO
